@@ -19,7 +19,7 @@ from datetime import datetime, timedelta
 
 import hashlib
 
-def pair_handling_message(fetcher, msg, compute_signal=True, compute_parallel = False):
+def pair_handling_message(fetcher, msg, compute_minute_signal=True, compute_day_signal=True, compute_parallel = False):
     high_dict = fetcher.pair_dict['high']
     low_dict = fetcher.pair_dict['low']
     volu_dict = fetcher.pair_dict['volu']
@@ -68,6 +68,12 @@ def pair_handling_message(fetcher, msg, compute_signal=True, compute_parallel =
             else:
                 fetcher.day_np = np.vstack((fetcher.day_np, day_new_data))
 
+            if compute_day_signal:
+                if compute_parallel:
+                    fetcher.launchDayParallelPool()
+                else:
+                    fetcher.launchDaySequential()
+
             if (len(fetcher.day_np.shape) > 1 and len(fetcher.day_np) > 3):
                 print('last three days')
                 print(fetcher.day_np[-3:, :])
@@ -78,6 +84,12 @@ def pair_handling_message(fetcher, msg, compute_signal=True, compute_parallel =
                 fetcher.upload_last_day_24_hours_signals_dataframe()
                 fetcher.reset_hour_np_to_lookback_window()
 
+            if fetcher.day_np is not None and len(fetcher.day_np)>0:
+                print('persisting minutes dataframe for the previous hour')
+                #fetcher.upload_last_hour_60_minutes_dataframe()
+                fetcher.upload_days_signals_dataframe()
+
+
         # we clear every former minutes
         day_high_dict.clear()
         day_low_dict.clear()
@@ -167,7 +179,7 @@ def pair_handling_message(fetcher, msg, compute_signal=True, compute_parallel =
                 fetcher.minute_np = new_data
             else:
                 fetcher.minute_np = np.vstack((fetcher.minute_np, new_data))
-            if compute_signal:
+            if compute_minute_signal:
                 if compute_parallel:
                     fetcher.launchMinuteParallelPool()
                 else:
@@ -194,9 +206,13 @@ def pair_handling_message(fetcher, msg, compute_signal=True, compute_parallel =
     volu_dict[trade_time_unix_timestamp_minute] = actual_volu + float(msg['q'])
 
 class RealTimeSignalEnsemblingParalleLauncher():
-    def __init__(self, starting_date=None, running_date=None, drop_token='', dropbox_backup=True, BINANCE_PUBLIC='', BINANCE_SECRET='', AWS_PUBLIC='', AWS_SECRET='', AWS_REGION='',bucket='', local_root_directory='../data',underlying = None, pair = None, frequence='daily', selected_algo='',user='napoleon',  db_path_suffix = '_run.sqlite', list_pkl_file_suffix = 'my_list.pkl',freqly_return_pkl_filename_suffix='freqly_candels.pkl'):
-        self.starting_date = starting_date
-        self.running_date = running_date
+    def __init__(self, min_calib_starting_date=None, min_calib_running_date=None, day_calib_starting_date=None, day_calib_running_date=None, drop_token='', dropbox_backup=True, BINANCE_PUBLIC='', BINANCE_SECRET='', AWS_PUBLIC='', AWS_SECRET='', AWS_REGION='',bucket='', local_root_directory='../data',underlying = None, pair = None, min_calib_selected_algo='', day_calib_selected_algo='', user='napoleon',  db_path_suffix = '_run.sqlite', list_pkl_file_suffix = 'my_list.pkl'):
+        self.min_calib_starting_date = min_calib_starting_date
+        self.min_calib_running_date = min_calib_running_date
+
+        self.day_calib_starting_date = day_calib_starting_date
+        self.day_calib_running_date = day_calib_running_date
+
         self.pair = pair
         self.BINANCE_PUBLIC = BINANCE_PUBLIC
         self.BINANCE_SECRET = BINANCE_SECRET
@@ -217,10 +233,14 @@ class RealTimeSignalEnsemblingParalleLauncher():
         self.minute_df = pd.DataFrame(columns=self.generic_columns)
 
         self.hour_df = pd.DataFrame(columns=self.generic_columns)
+        self.day_df = pd.DataFrame(columns=self.generic_columns)
 
         self.minute_np = None
         self.minute_signal_np = None
         self.hour_np = None
+        self.day_np = None
+        self.day_signal_np = None
+
         self.pair_dict = {}
         self.pair_dict['high'] = {}
         self.pair_dict['low'] = {}
@@ -239,42 +259,64 @@ class RealTimeSignalEnsemblingParalleLauncher():
         self.pair_dict['day_close'] = {}
         self.pair_dict['day_open'] = {}
 
-        self.args = []
-        self.counter = 1
         self.seed = 0
         self.local_root_directory = local_root_directory
 
         self.underlying = underlying
+
         self.list_pkl_file_suffix = list_pkl_file_suffix
-        self.frequence=frequence
-        self.selected_algo=selected_algo
 
-        self.dates_stub = self.starting_date.strftime('%d_%b_%Y') + '_' + self.running_date.strftime('%d_%b_%Y')
+        self.min_calib_selected_algo=min_calib_selected_algo
+        self.day_calib_selected_algo = day_calib_selected_algo
+
+        self.min_calib_dates_stub = self.min_calib_starting_date.strftime('%d_%b_%Y') + '_' + self.min_calib_running_date.strftime('%d_%b_%Y')
+        self.min_calib_list_pkl_file_name = self.min_calib_dates_stub + '_' + self.underlying + '_minutely_' + self.min_calib_selected_algo + self.list_pkl_file_suffix
+
+        self.day_calib_dates_stub = self.day_calib_starting_date.strftime('%d_%b_%Y') + '_' + self.day_calib_running_date.strftime('%d_%b_%Y')
+        self.day_calib_list_pkl_file_name = self.day_calib_dates_stub + '_' + self.underlying + '_daily_' + self.day_calib_selected_algo + self.list_pkl_file_suffix
+
+        print('minutes_selected algos')
+        print(self.min_calib_list_pkl_file_name)
+
+        print('days_selected algos')
+        print(self.day_calib_list_pkl_file_name)
 
-        self.list_pkl_file_name = self.dates_stub + '_' + self.underlying + '_' + self.frequence + '_' + self.selected_algo + self.list_pkl_file_suffix
-        print('selected algos')
-        print(self.list_pkl_file_name)
         if self.dropbox_backup:
-            self.signals_list =  self.dbx.download_pkl(self.list_pkl_file_name)
+            self.min_calib_signals_list =  self.dbx.download_pkl(self.min_calib_list_pkl_file_name)
+            self.day_calib_signals_list =  self.dbx.download_pkl(self.day_calib_list_pkl_file_name)
+
         else:
-            self.signals_list = napoleon_connector.load_pickled_list(local_root_directory=self.local_root_directory,
-                                                                 list_pkl_file_name=self.list_pkl_file_name)
-        self.signal_mapping={}
-        max_lookback_window = 0
-        for me_signal in self.signals_list:
-            self.args.append((me_signal))
-            self.counter = self.counter + 1
+            self.min_calib_signals_list = napoleon_connector.load_pickled_list(local_root_directory=self.local_root_directory,
+                                                                               list_pkl_file_name=self.min_calib_list_pkl_file_name)
+            self.day_calib_signals_list = napoleon_connector.load_pickled_list(local_root_directory=self.local_root_directory,
+                                                                               list_pkl_file_name=self.day_calib_list_pkl_file_name)
+
+        self.minute_signal_mapping={}
+        minute_max_lookback_window = 0
+        for me_signal in self.min_calib_signals_list:
             run_json_string = signal_utility.recover_to_sql_column_format(me_signal)
             params = json.loads(run_json_string)
             salty = str(int(hashlib.sha1(run_json_string.encode('utf-8')).hexdigest(), 16) % (10 ** 8))
             params = json.loads(run_json_string)
             readable_label = params['signal_type'] + str(params['trigger']) + salty
-            self.signal_mapping[me_signal]=readable_label
-            if params['lookback_window']>max_lookback_window:
-                max_lookback_window=params['lookback_window']
-        self.max_lookback_window = max_lookback_window
+            self.minute_signal_mapping[me_signal]=readable_label
+            if params['lookback_window']>minute_max_lookback_window:
+                minute_max_lookback_window=params['lookback_window']
+        self.minute_max_lookback_window = minute_max_lookback_window
+
+        self.day_signal_mapping={}
+        day_max_lookback_window = 0
+        for me_signal in self.day_calib_signals_list:
+            run_json_string = signal_utility.recover_to_sql_column_format(me_signal)
+            params = json.loads(run_json_string)
+            salty = str(int(hashlib.sha1(run_json_string.encode('utf-8')).hexdigest(), 16) % (10 ** 8))
+            params = json.loads(run_json_string)
+            readable_label = params['signal_type'] + str(params['trigger']) + salty
+            self.day_signal_mapping[me_signal]=readable_label
+            if params['lookback_window']>day_max_lookback_window:
+                day_max_lookback_window=params['lookback_window']
+        self.day_max_lookback_window = day_max_lookback_window
 
-        self.args.sort()
         self.user = user
         self.db_path_suffix = db_path_suffix
         self.filename =  user + db_path_suffix
@@ -283,10 +325,8 @@ class RealTimeSignalEnsemblingParalleLauncher():
         self.totalRow = 0
         self.empty_runs_to_investigate = []
         self.dbx = dropbox_file_saver.NaPoleonDropboxConnector(drop_token=drop_token,dropbox_backup=dropbox_backup)
-        self.freqly_return_pkl_filename_suffix=freqly_return_pkl_filename_suffix
-        self.saving_return_path = self.local_root_directory+self.underlying+self.freqly_return_pkl_filename_suffix
-
         self.last_minute = None
+        self.last_day = None
 
 
     def get_aws_last_past_minutes(self, nb_last_hours = 24):
@@ -400,6 +440,28 @@ class RealTimeSignalEnsemblingParalleLauncher():
         print(self.minute_np.shape)
         print(self.minute_signal_np.shape)
 
+    def reset_hour_np_to_lookback_window(self, loolback_window = 70):
+        print('hours shape before day reset')
+        print(self.hour_np.shape)
+        self.hour_np = self.hour_np[-loolback_window:]
+        print('hours shape after day reset')
+        print(self.hour_np.shape)
+
+    def update_day_dataframe(self):
+        print('updating day df')
+        if len(self.day_np.shape)>1:
+            updated_df=pd.DataFrame(data=self.day_np, columns = self.generic_columns, index=range(len(self.day_np)))
+        else:
+            updated_df=pd.DataFrame(data=self.day_np.reshape(1,len(self.day_np)), columns = self.generic_columns, index=[0])
+        updated_df['ts'] = updated_df['ts'].astype(int)
+        updated_df['ts'] = pd.to_datetime(updated_df['ts'], unit='s')
+        updated_df = updated_df.sort_values(by=['ts'])
+        updated_df = updated_df.rename(columns={"ts": "date"}, errors="raise")
+        updated_df = updated_df.set_index(updated_df['date'])
+        updated_df = updated_df.drop(columns=['date'])
+        self.day_df = updated_df
+        self.last_day = max(self.day_df.index)
+
     def update_minute_dataframe(self):
         print('updating minute df')
         if len(self.minute_np.shape)>1:
@@ -415,16 +477,10 @@ class RealTimeSignalEnsemblingParalleLauncher():
         self.minute_df = updated_df
         self.last_minute = max(self.minute_df.index)
 
-    def reset_hour_np_to_lookback_window(self, loolback_window = 70):
-        print('hours shape before day reset')
-        print(self.hour_np.shape)
-        self.hour_np = self.hour_np[-loolback_window:]
-        print('hours shape after day reset')
-        print(self.hour_np.shape)
+
 
     def upload_last_day_24_hours_signals_dataframe(self):
         print('updating 24 hours day df')
-
         updated_df=pd.DataFrame(data=self.hour_np, columns = self.generic_columns, index=range(len(self.minute_signal_np)))
         # updated_df = updated_df.astype(
         #     {'ts': 'float64', 'open': 'float64', 'high': 'float64', 'low': 'float64', 'close': 'float64','volumefrom': 'float64'}
@@ -446,13 +502,15 @@ class RealTimeSignalEnsemblingParalleLauncher():
         updated_df = updated_df.drop(columns=['date'])
         self.hour_df = updated_df
         local_path = self.local_root_directory + filename
-        self.minute_df.to_csv(local_path)
+        self.hour_df.to_csv(local_path)
+        print('uploading last day file '+local_path)
         self.aws_client.upload_file(self.bucket,local_path)
 
     def upload_last_hour_60_minutes_signals_dataframe(self):
         print('updating 60 minutes hour df')
-        updated_df=pd.DataFrame(data=self.minute_np, columns = self.generic_columns, index=range(len(self.minute_np)))
-        #updated_df = updated_df.astype({'ts': 'float32','open': 'float32', 'high': 'float32', 'low': 'float32', 'close': 'float32', 'volumefrom': 'float32'})
+        signal_columns = [self.minute_signal_mapping[me_sig] for me_sig in self.min_calib_signals_list]
+        me_columns = self.generic_columns + signal_columns
+        updated_df=pd.DataFrame(data=self.minute_signal_np, columns = me_columns, index=range(len(self.minute_signal_np)))
         updated_df['ts']=updated_df['ts'].astype(int)
         updated_df['hour_ts'] = updated_df['ts']-updated_df['ts']%3600
         updated_df = updated_df.astype({'hour_ts': 'int64'})
@@ -469,26 +527,85 @@ class RealTimeSignalEnsemblingParalleLauncher():
         self.minute_df = updated_df
         local_path = self.local_root_directory + filename
         self.minute_df.to_csv(local_path)
+        print('uploading last hour file '+local_path)
         self.aws_client.upload_file(self.bucket,local_path)
 
+    def upload_days_signals_dataframe(self):
+        print('updating day df')
+        signal_columns = [self.day_signal_mapping[me_sig] for me_sig in self.day_calib_signals_list]
+        me_columns = self.generic_columns + signal_columns
+        updated_df=pd.DataFrame(data=self.day_signal_np, columns = me_columns, index=range(len(self.day_signal_np)))
+        updated_df['ts']=updated_df['ts'].astype(int)
+        updated_df['hour_ts'] = updated_df['ts']-updated_df['ts']%3600
+        updated_df['day_ts'] = updated_df['ts']-updated_df['ts']%(3600*24)
+        updated_df = updated_df.astype({'day_ts': 'int64'})
+        if updated_df['day_ts'].nunique()>1:
+            print('too many days saved together : investigate')
+        day_timestamp = updated_df['day_ts'].max()
+        day_dt_object = datetime.utcfromtimestamp(day_timestamp)
+        filename = self.pair+'_'+day_dt_object.strftime('%d_%b_%Y') #str(hourr_timestamp)
+        updated_df['ts'] = pd.to_datetime(updated_df['ts'], unit='s')
+        updated_df = updated_df.sort_values(by=['ts'])
+        updated_df = updated_df.rename(columns={"ts": "date"}, errors="raise")
+        updated_df = updated_df.set_index(updated_df['date'])
+        updated_df = updated_df.drop(columns=['date'])
+        self.day_df = updated_df
+        local_path = self.local_root_directory + filename
+        self.day_df.to_csv(local_path)
+        self.aws_client.upload_file(self.bucket,local_path)
+        return
+
     def download_minute_dataframe(self, filename):
         dataframe = self.aws_client.download_dataframe_from_csv(self.bucket, filename)
         return dataframe
 
-    def launchMinuteParallelPool(self, use_num_cpu):
-        print('launching parallel computation for alphas at '+str(self.last_minute))
+    def launchDayParallelPool(self, use_num_cpu):
+        self.update_day_dataframe()
+        print('launching parallel computation for alphas at '+str(self.last_day))
         with Pool(processes=use_num_cpu) as pool:
-            run_results = pool.starmap(self.runMinuteSignal, self.args)
+            run_results = pool.starmap(self.runDaySignal, self.day_calib_signals_list)
         print('parallel computation done')
         print('results length')
         print(len(run_results))
 
+    def launchDaySequential(self):
+        self.update_day_dataframe()
+        print('launching sequential computation for alphas at '+str(self.last_day))
+        run_results = []
+        for meArg in self.day_calib_signals_list:
+            run_results.append(self.runDaySignal(meArg))
+        print('results length')
+        print(len(run_results))
+        new_data_sig = np.array(run_results)
+        if len(self.day_np.shape)>1:
+            new_data_sig = np.hstack((self.day_np[-1,:].reshape(1, len(self.generic_columns)), new_data_sig.reshape(1, len(new_data_sig))))
+        else:
+            new_data_sig = np.hstack((self.day_np.reshape(1, len(self.generic_columns)), new_data_sig.reshape(1, len(new_data_sig))))
+        print('new daily data signal')
+        print(new_data_sig)
+        if self.day_signal_np is None:
+            self.day_signal_np = new_data_sig
+        else:
+            self.day_signal_np = np.vstack((self.day_signal_np, new_data_sig))
+        if len(self.day_signal_np.shape)>1 and len(self.day_signal_np)>3:
+            print('last three daily signals')
+            print(self.day_signal_np[-3:])
+
+
+    def launchMinuteParallelPool(self, use_num_cpu):
+        self.update_minute_dataframe()
+        print('launching parallel computation for alphas at '+str(self.last_minute))
+        with Pool(processes=use_num_cpu) as pool:
+            run_results = pool.starmap(self.runMinuteSignal, self.min_calib_signals_list)
+        print('parallel computation done')
+        print('results length')
+        print(len(run_results))
 
     def launchMinuteSequential(self):
         self.update_minute_dataframe()
         print('launching sequential computation for alphas at '+str(self.last_minute))
         run_results = []
-        for meArg in self.args:
+        for meArg in self.min_calib_signals_list:
             run_results.append(self.runMinuteSignal(meArg))
         print('results length')
         print(len(run_results))
@@ -527,10 +644,30 @@ class RealTimeSignalEnsemblingParalleLauncher():
                                                                        data=x, **params))
         return last_generated_signal
 
+    def runDaySignal(self, me_signal):
+        ## idiosyncratic run itself
+        run_json_string = signal_utility.recover_to_sql_column_format(me_signal)
+        params = json.loads(run_json_string)
+        signal_type = params['signal_type']
+        normalization = params['normalization']
+        trigger = params['trigger']
+        transaction_costs = params['transaction_costs']
+        if normalization and not signal_generator.is_signal_continuum(signal_type):
+            return
+        lookback_window = params['lookback_window']
+        if lookback_window>self.minute_df.shape[0]:
+            return np.nan
+        freqly = self.day_df.iloc[-lookback_window:,:].copy()
+        signal_generation_method_to_call = getattr(signal_generator, signal_type)
+        last_generated_signal = signal_utility.compute_last_signal(freqly, lookback_window,
+                                                                   lambda x: signal_generation_method_to_call(
+                                                                       data=x, **params))
+        return last_generated_signal
+
 
 
     def hour_day_fetcher(self):
-        dfs_list = self.get_aws_last_passed_full_hours(nb_last_hours = 24)
+        dfs_list = self.get_aws_last_past_minutes(nb_last_hours = 24)
         dfs_list['ts'] = dfs_list.index
         # dfs_list['day_ts'] = dfs_list.ts.apply(lambda x: datetime.fromtimestamp(x).date())
         dfs_list['day_ts'] = dfs_list.ts.apply(lambda x: x.date())
