@@ -1446,3 +1446,67 @@ def fourier_hht_lo(data=None, threshold_1=0.05, threshold_2=0.5, alpha=0.05, n_i
             return np.nan
     except:
         return np.nan
+
+
+def superSmootherFilter(x, N):
+    n = len(x)
+    Filt = [0 for i in range(n)]
+    a1 = np.exp(np.sqrt(2) * np.pi / N)
+    b1 = 2 * a1 * np.cos(np.sqrt(2) * np.pi / N);
+    c2 = b1
+    c3 = -a1 * a1
+    c1 = 1 - c2 - c3
+    for i in range(2, len(x)):
+        Filt[i] = c1 * (x[i] + x[i - 1]) / 2 + c2 * Filt[i - 1] + c3 * Filt[i - 2]
+    return Filt
+
+
+def rooftingFilter(x, N):
+    alpha1 = 0
+    facteur = 1 / np.sqrt(2)
+    n = len(x)
+    HP = [0 for i in range(n)]
+    Filt = [0 for i in range(n)]
+
+    # High Pass Filter
+
+    alpha1 = (np.cos(facteur * 2 * np.pi / 48) + np.sin(facteur * 2 * np.pi / 48) - 1) / np.cos(
+        facteur * 2 * np.pi / 48)
+    for i in range(2, len(x)):
+        HP[i] = (1 - alpha1 / 2) * (1 - alpha1 / 2) * (x[i] - 2 * x[i - 1] + x[i - 2]) + 2 * (1 - alpha1) * HP[
+            i - 1] - (1 - alpha1) * (1 - alpha1) * HP[i - 2]
+    # Super Smooth Filter
+    rooftFilt = superSmootherFilter(HP, N)
+    return HP
+
+
+def rooftingFilter_lo(data=None, period=10, n_harm=3, fft_filter_type='low', contravariant=1, **kwargs):
+    n_predict = 1
+    try:
+        cur_df = data.copy()
+        x = cur_df['close'].values
+        approx = np.array(rooftingFilter(x, period))
+        last_available_close = approx[-1]
+        lb_signal = fourier_extrapolation(approx, n_predict, n_harm, fft_filter_type)
+        next_close = lb_signal[0]
+        test = contravariant * np.sign(next_close - last_available_close)
+        if test > 0:
+            return 1
+        else:
+            return np.nan
+    except:
+        return np.nan
+
+def rooftingFilter_ls(data=None, period=10, n_harm=3, fft_filter_type='low', contravariant=1, **kwargs):
+    n_predict = 1
+    try:
+        cur_df = data.copy()
+        x = cur_df['close'].values
+        approx = np.array(rooftingFilter(x, period))
+        last_available_close = approx[-1]
+        lb_signal = fourier_extrapolation(approx, n_predict, n_harm, fft_filter_type)
+        next_close = lb_signal[0]
+        test = contravariant * np.sign(next_close - last_available_close)
+        return test
+    except:
+        return np.nan
\ No newline at end of file
